"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bsv_minimal_1 = require("bsv-minimal");
const crypto_1 = __importDefault(require("crypto"));
const address_1 = __importDefault(require("./address"));
const config_1 = require("../config");
const { BufferReader, BufferWriter } = bsv_minimal_1.utils;
const VERSION_OBJ = {
    // version: 70001,
    // services: Buffer.alloc(8, 0),
    // version: VERSION,
    services: Buffer.from("0000000000000000", "hex"),
    // services: new BN(0),
    // timestamp: ,
    addr_recv: {
        services: Buffer.alloc(8, 0),
        // services: new BN(0),
        ip: Buffer.alloc(16, 0),
        port: 0,
    },
    addr_from: {
        services: Buffer.alloc(8, 0),
        // services: new BN(0),
        ip: Buffer.alloc(16, 0),
        port: 0,
    },
    nonce: crypto_1.default.randomBytes(8),
    // user_agent: ,
    // start_height: 0,
    relay: Buffer.from([1]), // Receive mempool txs
};
function read(payload) {
    let br = payload;
    if (Buffer.isBuffer(br))
        br = new BufferReader(br);
    // We don't NEED to explicityly type this, but it'll
    // be helpful so others can re-use the interface
    const result = {
        version: br.readUInt32LE(),
        // services : br.readUInt64LE(,
        services: br.readReverse(8),
        timestamp: br.readUInt64LE(),
        addr_recv: address_1.default.read(br, true),
        addr_from: address_1.default.read(br, true),
        nonce: br.read(8),
        user_agent: br.readVarLengthBuffer().toString(),
        start_height: br.readUInt32LE(),
        relay: br.readUInt8(),
    };
    // if (!br.eof()) throw new Error(`Invalid payload`)
    return result;
}
function write({ ticker, user_agent: userAgent, start_height: startHeight, mempoolTxs, version: versionParam, options, }) {
    options = Object.assign(Object.assign(Object.assign({}, VERSION_OBJ), { user_agent: userAgent, start_height: startHeight, version: versionParam, relay: mempoolTxs ? Buffer.from([1]) : Buffer.from([0]) }), options);
    let { version = config_1.VERSIONS[ticker] || config_1.VERSIONS.DEFAULT, services = VERSION_OBJ.services, timestamp = BigInt(Math.round(+new Date() / 1000)), addr_recv = VERSION_OBJ.addr_recv, addr_from = VERSION_OBJ.addr_from, nonce = VERSION_OBJ.nonce, user_agent = config_1.USER_AGENTS[ticker] || config_1.USER_AGENTS.DEFAULT, start_height = 0, relay = VERSION_OBJ.relay, } = options;
    const bw = new BufferWriter();
    bw.writeUInt32LE(version);
    // bw.writeUInt64LE(services)
    bw.writeReverse(services);
    bw.writeUInt64LE(timestamp);
    bw.write(address_1.default.write(addr_recv));
    bw.write(address_1.default.write(addr_from));
    bw.write(nonce);
    bw.writeVarintNum(Buffer.from(user_agent).length);
    bw.write(Buffer.from(user_agent));
    bw.writeUInt32LE(start_height);
    bw.write(relay);
    return bw.toBuffer();
}
const Version = {
    read,
    write,
};
exports.default = Version;
//# sourceMappingURL=version.js.map